#!/usr/bin/env python3
############################################################################
# Copyright (C) 2020 Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""
Summarize validation results.

The current version of this script expects to be able to
use standard Operating System ways of traversing a Release
artifact tree. This can be accomplished by mounting the
sharepoint folder in OneDrive, or by downloading a copy
of the folder.

Supported test formats (auto-detected):
XML (*.xml):
    Robot Framework XML Format
    jUnit XML Format (xUnit)

CSV (*.csv):
    Teamcity Test Result CSV
    Grid Report CSV
    General CSV
        Delimiter may be:
            ',' (comma), ';' (semi-colon), or '\\t' (tab)
        (optional) Test Suite Column may be:
            'suite', 'suite name', or 'test suite'
        Test Name Column may be:
            'test', 'test name', 'test case', or 'case'
        Result Column may be:
            'status', or 'result'

        Test name is: "<Test Suite>.<Test Name>" if Test Suite
        is set, otherwise "<Test Name>"

        Passing Results are (case insensitive):
            'pass', 'passed', or 'ok'

        Failing Results are (case insensitive):
            'fail', 'failed', or 'error'

        Skipped Results are (case insensitive):
            'skip', 'skipped', 'ignore', or 'ignored'

The defect database file is formatted as json in the form:
    {
        "Package Name" : {
            "Jira ID" : [
                "Test Name",
            ]
        },
    }

    for example:

    {
        "msdk_compatibility" : {
            "VPL7-1762" : [
                "hevce_10b_420_i010_reset.30",
                "hevce_8b_420_iyuv_reset.2",
                "hevce_8b_420_iyuv_reset.30"
            ]
        },
        "operating systems" : {
            "VPL7-3005" : [
                "oneVPL-cpu.Pipeline.1toN.Fused.Decvpp.H264.to I420 & BGRA system memory"
            ],
            "VPL7-3072" : [
                "oneVPL-gen.Decode.H264.Handles corrupted data",
                "oneVPL-gen.Decode.H264.I420"
            ]
        }
    }

"""
# pylint: disable=too-many-lines
import argparse
from argparse import RawTextHelpFormatter
from contextlib import contextmanager
from glob import glob
import os
import sys
import csv
import json
from xml.etree.ElementTree import SubElement  # nosec
from xml.etree.ElementTree import Element  # nosec
import xlsxwriter
from defusedxml import minidom
import defusedxml.ElementTree as DET

SCRIPT_PATH = os.path.realpath(
    os.path.join(os.getcwd(), os.path.dirname(__file__)))


def pretty_print_xml(root):
    """Format xml for legibility"""
    serial_xml = DET.tostring(root)
    parsed_xml = minidom.parseString(serial_xml)
    return parsed_xml.toprettyxml()


@contextmanager
def pushd(dest):
    """change working directory"""
    cur_dir = os.getcwd()
    os.chdir(dest)
    try:
        yield
    finally:
        os.chdir(cur_dir)


INDENT_LEVEL = 0
INDENT = ""


# pylint: disable=global-statement
@contextmanager
def log_scope():
    """Add a scope level to the logging stack"""
    global INDENT_LEVEL
    global INDENT
    INDENT_LEVEL += 1
    INDENT = "\t" * INDENT_LEVEL
    yield
    INDENT_LEVEL -= 1
    INDENT = "\t" * INDENT_LEVEL


def log(msg: str):
    """Log a message, indenting all lines based on the log scope"""
    for line in msg.splitlines():
        print(f"{INDENT}{line}")


def log_summary(name, summary):
    """Log the summary of a test suite"""
    unique = [
        summary.unique_attempt_count, summary.unique_run_count,
        summary.unique_pass_count, summary.unique_fail_count,
        summary.unique_skip_count
    ]
    unique = [str(value) for value in unique]
    total = [
        summary.total_attempt_count, summary.total_run_count,
        summary.total_pass_count, summary.total_fail_count,
        summary.total_skip_count
    ]
    total = [str(value) for value in total]
    if name is not None:
        log(name)
    log("       Attempt\tRun\tPass\tFail\tSkip")
    log(" Total:\t" + ("\t".join(total)))
    log("Unique:\t" + ("\t".join(unique)))


class TestResult:
    """Results of running a test (possibly more than once)"""
    def __init__(self, name):
        self.classname = None
        self.name = name
        self.runs = 0
        self.passes = 0
        self.fails = 0
        self.skips = 0
        self._dirty = True

    def clean(self):
        """Clear "dirty" flag for summary caching"""
        self._dirty = False

    @property
    def is_dirty(self):
        """Check "dirty" flag for summary caching"""
        return self._dirty

    def squash(self):
        """Rewrite result to look like a single test execution"""
        if self.fails:
            self.runs = 1
            self.passes = 0
            self.fails = 1
            self.skips = 0
        elif self.passes:
            self.runs = 1
            self.passes = 1
            self.fails = 0
            self.skips = 0
        elif self.skips:
            self.runs = 0
            self.passes = 0
            self.fails = 0
            self.skips = 1
        self._dirty = True

    def fill_xunit(self, suite_xml, test_class=None):
        """Add test result to an XML element in xUnit format"""
        for _ in range(self.passes):
            testcase = SubElement(suite_xml, "testcase")
            testcase.attrib["name"] = str(self.name)
            testcase.attrib["status"] = "Pass"
            if test_class is not None:
                testcase.attrib["classname"] = str(test_class)

        for _ in range(self.fails):
            testcase = SubElement(suite_xml, "testcase")
            testcase.attrib["name"] = str(self.name)
            if test_class is not None:
                testcase.attrib["classname"] = str(test_class)
            testcase.attrib["status"] = "Fail"
            failure = SubElement(testcase, "failure")
            failure.attrib["type"] = "Test Failed"

        for _ in range(self.skips):
            testcase = SubElement(suite_xml, "testcase")
            if test_class is not None:
                testcase.attrib["classname"] = str(test_class)
            testcase.attrib["status"] = "Skip"
            testcase.attrib["name"] = str(self.name)
            skipped = SubElement(testcase, "skipped")
            skipped.text = "Skipped"

    @property
    def pass_count(self):
        """Number of passes recorded"""
        return self.passes

    @property
    def fail_count(self):
        """Number of fails recorded"""
        return self.fails

    @property
    def skip_count(self):
        """Number of skips recorded"""
        return self.skips

    @property
    def run_count(self):
        """Number of runs recorded"""
        return self.runs

    @property
    def attempt_count(self):
        """Number of attempts recorded"""
        return self.runs + self.skips

    def update(self, other):
        """Add results from another test result"""
        self.runs += other.runs
        self.passes += other.passes
        self.fails += other.fails
        self.skips += other.skips
        self._dirty = True

    def add_pass(self):
        """Record a pass"""
        self.passes += 1
        self.runs += 1
        self._dirty = True
        return self

    def add_skip(self):
        """Record a skip"""
        self.skips += 1
        self._dirty = True
        return self

    def add_fail(self):
        """Record a fail"""
        self.fails += 1
        self.runs += 1
        self._dirty = True
        return self

    @property
    def unique_pass_count(self):
        """Report if all recorded results should be rolled up as a pass
        Reported as 1 or 0 to simplify accumulator logic higher in the stack
        """
        if self.runs > 0 and self.fails == 0:
            return 1
        return 0

    @property
    def unique_fail_count(self):
        """Report if all recorded results should be rolled up as a fail
        Reported as 1 or 0 to simplify accumulator logic higher in the stack
        """
        if self.fails > 0:
            return 1
        return 0

    @property
    def unique_skip_count(self):
        """Report if all recorded results should be rolled up as a skip
        Reported as 1 or 0 to simplify accumulator logic higher in the stack
        """
        if self.skips > 0 and self.runs == 0:
            return 1
        return 0

    @property
    def unique_run_count(self):
        """Report if all recorded results should be rolled up as a run
        Reported as 1 or 0 to simplify accumulator logic higher in the stack
        """
        if self.runs > 0:
            return 1
        return 0

    @property
    def unique_attempt_count(self):
        """Report if all recorded results should be rolled up as an attempt
        Reported as 1 or 0 to simplify accumulator logic higher in the stack
        """
        if self.skips > 0 or self.runs > 0:
            return 1
        return 0


class TestContainer:
    """A collection of test results"""
    def __init__(self):
        self.tests = {}
        self._dirty = True

    def clean(self):
        """Clear "dirty" flag for summary caching"""
        self._dirty = False
        for test in self.tests.values():
            test.clean()

    @property
    def is_dirty(self):
        """Check "dirty" flag for summary caching"""
        if self._dirty:
            return True
        for test in self.tests.values():
            if test.is_dirty:
                return True
        return False

    def _get_test(self, name):
        """find or create a test by name"""
        if name not in self.tests:
            self._dirty = True
            test = TestResult(name)
            self.tests[name] = test
        return self.tests[name]

    @property
    def unique_pass_count(self):
        """Report number of cases where for all tests with
        the same name all tests are passing"""
        count = 0
        for test in self.tests.values():
            count += test.unique_pass_count
        return count

    @property
    def unique_fail_count(self):
        """Report number of cases where for all tests with
        the same name at least one test is failing"""
        count = 0
        for test in self.tests.values():
            count += test.unique_fail_count
        return count

    @property
    def unique_skip_count(self):
        """Report number of cases where for all tests with
        the same name all tests are skipped"""
        count = 0
        for test in self.tests.values():
            count += test.unique_skip_count
        return count

    @property
    def unique_run_count(self):
        """Report number of cases where for all tests with
        the same name all tests not tests are skipped"""
        count = 0
        for test in self.tests.values():
            count += test.unique_run_count
        return count

    @property
    def unique_attempt_count(self):
        """Report number of cases where for all tests with
        the same name all tests have been run or skipped."""
        count = 0
        for test in self.tests.values():
            count += test.unique_attempt_count
        return count

    @property
    def total_pass_count(self):
        """Report the total count of passes recorded for all tests"""
        count = 0
        for test in self.tests.values():
            count += test.pass_count
        return count

    @property
    def total_fail_count(self):
        """Report the total count of fails recorded for all tests"""
        count = 0
        for test in self.tests.values():
            count += test.fail_count
        return count

    @property
    def total_skip_count(self):
        """Report the total count of skips recorded for all tests"""
        count = 0
        for test in self.tests.values():
            count += test.skip_count
        return count

    @property
    def total_run_count(self):
        """Report the total count of runs recorded for all tests"""
        count = 0
        for test in self.tests.values():
            count += test.run_count
        return count

    @property
    def total_attempt_count(self):
        """Report the total count of attempts recorded for all tests"""
        count = 0
        for test in self.tests.values():
            count += test.attempt_count
        return count

    @property
    def test_count(self):
        """Report the total count tests recorded"""
        count = len(self.tests)
        return count

    def add_pass(self, name):
        """record a test passing"""
        return self._get_test(name).add_pass()

    def add_skip(self, name):
        """record a test skipped"""
        return self._get_test(name).add_skip()

    def add_fail(self, name):
        """record a test failing"""
        return self._get_test(name).add_fail()

    def update(self, other):
        """Add results from another test collection"""
        for name, test_result in other.tests.items():
            result = self._get_test(name)
            result.update(test_result)


class CIDictReader(csv.DictReader):
    """A Case insensitive DictReader. All names will be casefolded (aggressively lowercased)"""
    @property
    def fieldnames(self):
        """Access field names"""
        return [
            field.strip().casefold()
            for field in csv.DictReader.fieldnames.fget(self)
        ]

    # pylint: disable=no-member
    def next(self):
        """Iterate"""
        return CIDictReader(csv.DictReader.next(self))


class TestSuite(TestContainer):
    """A collection of test results grouped together to represent a
    higher order result
    """
    def __init__(self, name, location):
        super().__init__()
        self.name = name
        self.location = location

    def composite_result(self):
        """merge all suites under this object into one suite
        (degenerate case)"""
        return self

    def squash(self, location):
        """Rewrite to represent a single execution of each test"""
        self.location = location
        for test in self.tests.values():
            test.squash()
        self._dirty = True

    def print_log(self):
        """Print summary to the log"""
        log_summary(self.name, self)

    def _read_robot_test(self, element, parent_name=None):
        """Read in results from a Robot Framework Test"""
        tests_found = 0
        test_name = element.attrib["name"]
        if parent_name:
            suite_name = parent_name + "." + test_name
        else:
            suite_name = test_name
        for status in element.findall("./status"):
            status_name = status.attrib["status"].lower()
            if status_name == "pass":
                tests_found += 1
                self.add_pass(suite_name)
            elif status_name == "skip":
                tests_found += 1
                self.add_skip(suite_name)
            else:
                tests_found += 1
                self.add_fail(suite_name)
        return tests_found

    def _read_robot_suite(self, element, parent_name=None):
        """Read in results from a Robot Framework Suite"""
        tests_found = 0
        suite_name = element.attrib["name"]
        if parent_name:
            suite_name = parent_name + "." + suite_name
        for test in element.findall("./test"):
            tests_found += self._read_robot_test(test, suite_name)
        for suite in element.findall("./suite"):
            tests_found += self._read_robot_suite(suite, suite_name)
        return tests_found

    def _read_robot_result(self, root):
        """Read in results from a Robot Framework xml file root element"""
        tests_found = 0
        for test in root.findall("./test"):
            tests_found += self._read_robot_test(test)
        for suite in root.findall("./suite"):
            tests_found += self._read_robot_suite(suite)
        return tests_found

    # pylint: disable=too-many-branches
    def _read_cdash_test(self, element):
        """Read in results from a CDash test report"""
        tests_found = 1
        test_status = element.attrib["Status"].lower()
        test_name = None
        if not test_name:
            name_el = element.find("Name")
            if name_el is not None:
                test_name = name_el.text
        if not test_name:
            name_el = element.find("FullName")
            if name_el is not None:
                test_name = name_el.text
        if not test_name:
            name_el = element.find("FullCommandLine")
            if name_el is not None and name_el.text:
                test_name = name_el.text
        if not test_name:
            test_name = "unnamed"
        if test_status == "passed":
            self.add_pass(test_name)
        elif test_status == "failed":
            self.add_fail(test_name)
        elif test_status == "Timeout":
            self.add_fail(test_name)
        elif test_status == "OTHER_FAULT":
            self.add_fail(test_name)
        elif test_status == "notrun":
            self.add_skip(test_name)
        elif test_status == "Disabled":
            self.add_skip(test_name)
        else:
            self.add_fail(test_name)
        return tests_found

    def _read_cdash_result(self, root):
        """Read in results from a CDash test report root element"""
        tests_found = 0
        for test in root.findall("./Testing/Test"):
            tests_found += self._read_cdash_test(test)
        return tests_found

    def _read_xunit_result(self, root):
        """Read in results from a JUnit xml file"""
        tests_found = 0
        for test_case in root.findall("testcase"):
            name = ""
            if "classname" in test_case.attrib:
                classname = test_case.attrib["classname"].split(".")
                if classname:
                    name += ".".join(classname) + "."
            name += test_case.attrib['name']

            failure = test_case.find("failure")
            if failure is not None:
                tests_found += 1
                self.add_fail(name)
                continue

            error = test_case.find("error")
            if error is not None:
                tests_found += 1
                self.add_fail(name)
                continue

            skip = test_case.find("skipped")
            if skip is not None:
                tests_found += 1
                self.add_skip(name)
                continue

            tests_found += 1
            self.add_pass(name)
        return tests_found

    def _read_csv_result(self, result: str):
        """Read in results from a TeamCity test format csv file"""
        tests_found = 0
        with open(result, newline='', encoding="utf8") as csv_file:
            try:
                dialect = csv.Sniffer().sniff(csv_file.readline(),
                                              delimiters=';,\t')
            except:
                print(f"Error reading {result}", file=sys.stderr)
                raise
            csv_file.seek(0)
            reader = CIDictReader(csv_file, dialect=dialect)
            for row in reader:

                name = ""
                for suite_col in ['suite', 'suite name', 'test suite']:
                    if suite_col in row:
                        name += row[suite_col]
                        break

                name += "." if name else ""

                for test_col in ['test', 'test name', 'test case', 'case']:
                    if test_col in row:
                        name += row[test_col]
                        break

                status = None
                for status_col in ['status', 'result']:
                    if status_col in row:
                        status = row[status_col].casefold()
                        break

                if name and status:
                    if status in ['pass', 'passed', 'ok']:
                        tests_found += 1
                        self.add_pass(name)
                    elif status in ['fail', 'failed', 'error']:
                        tests_found += 1
                        self.add_fail(name)
                    elif status in ['skip', 'skipped', 'ignore', 'ignored']:
                        tests_found += 1
                        self.add_skip(name)
        return tests_found

    def read_result(self, result: str, result_format: str):
        """Read in results from a file"""
        tests_found = 0
        if result_format == "xml":
            tree = DET.parse(result)
            root = tree.getroot()
            if root.tag == "robot":
                tests_found = self._read_robot_result(root)
            elif root.tag in ["testsuite", "testsuites", "testcase"]:
                tests_found = self._read_xunit_result(root)
            elif root.tag in ["Site", "NewDataSet"]:
                tests_found = self._read_cdash_result(root)
        elif result_format == "csv":
            tests_found = self._read_csv_result(result)
        return tests_found

    def fill_xunit(self, suites_xml, package=None):
        """Write out suite as JUnit format XML"""
        testsuite = SubElement(suites_xml, "testsuite")
        if package is not None:
            testsuite.attrib["package"] = str(package)
        if self.name is not None:
            testsuite.attrib["name"] = str(self.name)
            if package is None:
                testsuite.attrib["name"] = str(self.name)
            else:
                testsuite.attrib["name"] = str(package) + "/" + self.name
        elif package is not None:
            testsuite.attrib["name"] = str(package)
        summary = self
        testsuite.attrib["tests"] = str(summary.total_run_count)
        testsuite.attrib["failures"] = str(summary.total_fail_count)
        testsuite.attrib["skipped"] = str(summary.total_skip_count)
        for test in self.tests.values():
            test.fill_xunit(testsuite, test_class=package)


class TestPackage:
    """Intermediate level Container for test information.
    Collects Packages, that hold Suites, that hold Results"""
    def __init__(self, name):
        self.name = name
        self.suites = []
        self._composite = None
        self._dirty = True

    def clean(self):
        """Clear "dirty" flag for summary caching"""
        self._dirty = False
        for suite in self.suites:
            suite.clean()

    @property
    def is_dirty(self):
        """Check "dirty" flag for summary caching"""
        if self._dirty:
            return True
        for suite in self.suites:
            if suite.is_dirty:
                return True
        return False

    def composite_result(self):
        """merge all suites under this object into one suite"""
        if self.is_dirty:
            self._composite = TestSuite(self.name, None)
            for suite in self.suites:
                self._composite.update(suite)
        return self._composite

    def add_suite(self, suite: TestSuite):
        """Add a suite under this package"""
        self._dirty = True
        self.suites.append(suite)
        return suite

    def print_log(self):
        """Print summary to the log"""
        log_summary(self.name, self.composite_result())
        with log_scope():
            for suite in self.suites:
                suite.print_log()

    def scan_test_results(self,
                          source: str,
                          suite_name: str,
                          suite_per_file=False):
        """Read through all files in a folder to find test results"""
        test_result_files = []
        with pushd(source):
            test_result_files.extend([('xml', os.path.abspath(match))
                                      for match in glob("*.xml")])
            test_result_files.extend([('csv', os.path.abspath(match))
                                      for match in glob("*.csv")])
        count = 0
        suite = TestSuite(suite_name, source)

        for result_format, result_file in test_result_files:
            tests_found = None
            if suite_per_file:
                suite = TestSuite(
                    suite_name + "/" + os.path.basename(result_file),
                    result_file)
            tests_found = suite.read_result(result_file, result_format)
            if tests_found:
                count += 1
                if suite_per_file:
                    suite.squash(suite.location)
                    self.add_suite(suite)
        if not suite_per_file:
            if count > 0:
                suite.squash(source)
                self.add_suite(suite)
        return count

    def scan_tests(self,
                   folder_path: str,
                   suite_name: str = None,
                   suite_per_file=False):
        """Scan folder for test results."""
        with log_scope():
            self.scan_test_results(folder_path,
                                   suite_name,
                                   suite_per_file=suite_per_file)
            for folder in os.scandir(folder_path):
                if folder.is_dir():
                    if suite_name:
                        sub_suite = f'{suite_name}/{folder.name}'
                    else:
                        sub_suite = folder.name
                    self.scan_tests(folder.path,
                                    sub_suite,
                                    suite_per_file=suite_per_file)

    def fill_xunit(self, suites_xml):
        """Write out package as JUnit format XML"""
        for suite in self.suites:
            suite.fill_xunit(suites_xml, package=self.name)


class TestReport:
    """Top level container for test information.
    Collects Packages, that hold Suites, that hold Results"""
    def __init__(self):
        self.packages = []
        self._composite = None
        self._dirty = True

    def clean(self):
        """Clear "dirty" flag for summary caching"""
        self._dirty = False
        for package in self.packages:
            package.clean()

    @property
    def is_dirty(self):
        """Check "dirty" flag for summary caching"""
        if self._dirty:
            return True
        for package in self.packages:
            if package.is_dirty:
                return True
        return False

    def composite_result(self):
        """merge all suites under this object into one suite"""
        if self.is_dirty:
            self._composite = TestSuite("Report", None)
            for package in self.packages:
                self._composite.update(package.composite_result())
        return self._composite

    def add_package(self, name):
        """Add a package to the report"""
        result = TestPackage(name)
        self.packages.append(result)
        return result

    def print_log(self):
        """Print summary to the log"""
        log_summary(None, self.composite_result())
        with log_scope():
            for package in self.packages:
                package.print_log()

    def scan_tests(self, folder_path: str, suite_per_file=False):
        """Scan a folder for test results"""
        for folder in os.scandir(folder_path):
            if not folder.is_dir():
                continue
            test_suite = self.add_package(folder.name)
            test_suite.scan_tests(folder.path, suite_per_file=suite_per_file)

    def as_xunit(self):
        """Return the report represented as JUnit format XML"""
        suites_xml = Element('testsuites')
        summary = self.composite_result()
        suites_xml.attrib["tests"] = str(summary.total_run_count)
        suites_xml.attrib["failures"] = str(summary.total_fail_count)
        for package in self.packages:
            package.fill_xunit(suites_xml)
        return suites_xml


def get_suite_defects(summary, package_defects):
    """Get the set of defects that apply to a given suite"""
    result = set()
    for test_name in summary.tests:
        if summary.tests[test_name].fail_count > 0:
            found = False
            for defect, tests in package_defects.items():
                if test_name in tests:
                    found = True
                    result.add(defect)
            if not found:
                result.add("unknown")
    return result


# pylint: disable=too-many-locals,too-many-nested-blocks,too-many-branches
def write_excel_package_page(workbook: xlsxwriter.Workbook,
                             package: TestPackage,
                             report_all_tests=False,
                             defects=None):
    """Write an Excel tab with package details"""
    if defects is None:
        defects = {}
    suite_header_fmt = workbook.add_format()
    suite_header_fmt.set_rotation(90)
    suite_header_fmt.set_bold(True)
    header_fmt = workbook.add_format()
    header_fmt.set_bold(True)

    summary = package.composite_result()
    sheet = workbook.add_worksheet()
    sheet.name = package.name
    sheet.set_column("A:A", 9)
    sheet.write_string("A1", "Jira", cell_format=header_fmt)
    sheet.set_column("B:B", 70)
    sheet.write_string("B1", "Test", cell_format=header_fmt)
    f_row = 0
    f_col = 2
    for suite in package.suites:
        suite_name = suite.name
        if suite_name is None:
            suite_name = "<empty>"
        sheet.set_column(f_col, f_col, 2.8)
        sheet.write_string(f_row,
                           f_col,
                           suite_name,
                           cell_format=suite_header_fmt)
        f_col += 1
    width = f_col
    f_row += 1
    for test_name in summary.tests:
        if report_all_tests or summary.tests[test_name].fail_count > 0:
            f_col = 0
            jiras = set()
            for jira, tests in defects.items():
                if test_name in tests:
                    jiras.add(jira)
            sheet.write_string(f_row, f_col, " ".join(sorted(jiras)))
            f_col += 1
            sheet.write(f_row, f_col, test_name)
            f_col += 1
            for suite in package.suites:
                if test_name in suite.tests:
                    if report_all_tests:
                        if suite.tests[test_name].unique_fail_count:
                            sheet.write(f_row, f_col, "Fail")
                        elif suite.tests[test_name].unique_skip_count:
                            sheet.write(f_row, f_col, "Skip")
                        elif suite.tests[test_name].unique_pass_count:
                            sheet.write(f_row, f_col, "Pass")
                    elif test_name in suite.tests and suite.tests[
                            test_name].fail_count > 0:
                        sheet.write(f_row, f_col, "X")
                f_col += 1
            f_row += 1
    sheet.autofilter(0, 0, f_row - 1, width - 1)


def write_excel_dashboard_structure(workbook, dashboard):
    """Write summary dashboard high level page structure"""
    col_break_fmt = workbook.add_format()
    col_break_fmt.bg_color = '#F2F2F2'
    col_break_fmt.set_left(1)
    col_break_fmt.set_right(1)
    header_break_fmt = workbook.add_format()
    header_break_fmt.set_bold(True)
    header_break_fmt.set_bottom(1)
    header_col_break_fmt = workbook.add_format()
    header_col_break_fmt.bg_color = '#F2F2F2'
    header_col_break_fmt.set_left(1)
    header_col_break_fmt.set_right(1)
    header_col_break_fmt.set_bottom(1)
    shared_header_fmt = workbook.add_format()
    shared_header_fmt.set_align('center')
    shared_header_fmt.set_bold(True)
    bold_fmt = workbook.add_format()
    bold_fmt.set_bold(True)

    dashboard.set_column("A:A", 6, cell_format=bold_fmt)
    dashboard.set_column("B:C", 6.5)
    dashboard.set_column("D:D", 1.9, cell_format=col_break_fmt)
    dashboard.set_column("E:E", 20.15)
    dashboard.set_column("F:M", 4.45)
    dashboard.set_column("N:N", 1.9, cell_format=col_break_fmt)
    dashboard.set_column("O:O", 48.25)
    dashboard.set_column("P:S", 4.45)
    dashboard.set_row(1, None, cell_format=header_break_fmt)
    dashboard.write_blank("D2", None, cell_format=header_col_break_fmt)
    dashboard.write_blank("N2", None, cell_format=header_col_break_fmt)
    for cells, header in [
        ("A1:C1", "Summary"),
        ("F1:I1", "Total"),
        ("J1:M1", "Unique"),
    ]:
        dashboard.merge_range(cells, header, cell_format=shared_header_fmt)

    for col, header in [
        ("B", "Count"),
        ("C", "Rate"),
        ("E", "Category"),
        ("F", "Tests"),
        ("G", "Pass"),
        ("H", "Fail"),
        ("I", "Skip"),
        ("J", "Tests"),
        ("K", "Pass"),
        ("L", "Fail"),
        ("M", "Skip"),
        ("O", "Suite"),
        ("P", "Tests"),
        ("Q", "Pass"),
        ("R", "Fail"),
        ("S", "Skip"),
        ("T", "Jiras"),
    ]:
        dashboard.write_string(f"{col}2", header)


def write_excel_dashboard_summary(workbook, dashboard):
    """Write top level summary portion of dashboard"""
    footnote_fmt = workbook.add_format()
    footnote_fmt.set_font_size(8)
    footnote_fmt.set_align('left')
    footnote_fmt.set_align('top')
    rate_field_fmt = workbook.add_format()
    rate_field_fmt.set_num_format("0.00%")

    dashboard.write("A3", "Total")
    dashboard.write("A4", "Tests")
    dashboard.write_formula("B4", "=SUM(F:F)")
    dashboard.write("A5", "Pass*")
    dashboard.write_formula("B5", "=SUM(G:G)")
    dashboard.write_formula("C5", "=B5/SUM(B5:B6)", cell_format=rate_field_fmt)
    dashboard.write("A6", "Fail*")
    dashboard.write_formula("B6", "=SUM(H:H)")
    dashboard.write_formula("C6", "=B6/SUM(B5:B6)", cell_format=rate_field_fmt)
    dashboard.write("A7", "Skip")
    dashboard.write_formula("B7", "=SUM(I:I)")
    dashboard.write_formula("C7", "=B7/B4", cell_format=rate_field_fmt)
    dashboard.merge_range("A8:C8",
                          "*Rate excludes skipped",
                          cell_format=footnote_fmt)
    dashboard.write("A10", "Unique")
    dashboard.write("A11", "Tests")
    dashboard.write_formula("B11", "=SUM(J:J)")
    dashboard.write("A12", "Pass")
    dashboard.write_formula("B12", "=SUM(K:K)")
    dashboard.write_formula("C12",
                            "=B12/SUM(B12:B13)",
                            cell_format=rate_field_fmt)
    dashboard.write("A13", "Fail")
    dashboard.write_formula("B13", "=SUM(L:L)")
    dashboard.write_formula("C13",
                            "=B13/SUM(B12:B13)",
                            cell_format=rate_field_fmt)
    dashboard.write("A14", "Skip")
    dashboard.write_formula("B14", "=SUM(M:M)")
    dashboard.write_formula("C14", "=B14/B11", cell_format=rate_field_fmt)


def write_excel_package_summary(workbook, dashboard, package, row):
    """Write summary for one package on dashboard"""
    # workbook is passed to allow access to book wide information like formats
    del workbook
    summary = package.composite_result()
    dashboard.write(f"E{row}", package.name)
    dashboard.write(f"F{row}", summary.total_attempt_count)
    dashboard.write(f"G{row}", summary.total_pass_count)
    dashboard.write(f"H{row}", summary.total_fail_count)
    dashboard.write(f"I{row}", summary.total_skip_count)
    dashboard.write(f"J{row}", summary.unique_attempt_count)
    dashboard.write(f"K{row}", summary.unique_pass_count)
    dashboard.write(f"L{row}", summary.unique_fail_count)
    dashboard.write(f"M{row}", summary.unique_skip_count)


# pylint: disable=too-many-arguments,too-many-positional-arguments
def write_excel_suite_summary(workbook, dashboard, suite, row, target_folder,
                              package_defects):
    """Write summary for one suite on dashboard"""
    # workbook is passed to allow access to book wide information like formats
    del workbook
    summary = suite.composite_result()
    if suite.location is not None:
        location = os.path.relpath(suite.location, target_folder)
        location.replace("\\", "/")
        dashboard.write_url(f"O{row}",
                            f"external:{location}",
                            string=suite.name)
    else:
        dashboard.write(f"O{row}", suite.name)
    dashboard.write(f"P{row}", summary.total_attempt_count)
    dashboard.write(f"Q{row}", summary.total_pass_count)
    dashboard.write(f"R{row}", summary.total_fail_count)
    dashboard.write(f"S{row}", summary.total_skip_count)
    dashboard.write(f"T{row}",
                    ",".join(get_suite_defects(summary, package_defects)))


def write_excel(report: TestReport,
                target,
                report_all_tests=False,
                defects=None):
    """Write report as an Excel file"""
    if defects is None:
        defects = {}
    target = os.path.abspath(target)
    target_folder = os.path.dirname(target)
    workbook = xlsxwriter.Workbook(target)
    dash = workbook.add_worksheet()
    dash.name = "Dashboard"

    write_excel_dashboard_structure(workbook, dash)
    write_excel_dashboard_summary(workbook, dash)
    summary = report.composite_result()
    row = 3
    for package in report.packages:
        package_defects = defects.get(package.name, {})
        write_excel_package_summary(workbook, dash, package, row)
        row += 1
        for suite in package.suites:
            write_excel_suite_summary(workbook, dash, suite, row,
                                      target_folder, package_defects)
            row += 1
        if summary.total_fail_count > 0:
            write_excel_package_page(workbook,
                                     package,
                                     report_all_tests=report_all_tests,
                                     defects=package_defects)

    workbook.close()


def read_command_line(cmd_line):
    """
    Read command line arguments
    """
    # Now read full arguments
    parser = argparse.ArgumentParser(description=globals()['__doc__'],
                                     formatter_class=RawTextHelpFormatter)
    parser.add_argument('source',
                        help="Validation data dir",
                        type=os.path.abspath,
                        action='store')

    parser.add_argument('--defects',
                        help="Database of known defects",
                        type=os.path.abspath)

    parser.add_argument('--suite-per-file',
                        help="Treat each result file as a separate suite",
                        action='store_true')

    parser.add_argument('--report-all-tests',
                        help="List all tests in summary report",
                        action='store_true')

    # parse arguments (will exit here on invalid args or help)
    args = parser.parse_args(args=cmd_line)
    args.dest = args.source
    return args


def load_defects(db_path):
    """Load defect database"""
    if not db_path or not os.path.isfile(db_path):
        return {}
    with open(db_path, newline='', encoding="utf8") as db_file:
        json_data = json.load(db_file)
        return json_data


def run(args):
    """main entry point"""
    validation_summary_path = os.path.join(args.dest,
                                           "validation-summary.xlsx")
    xunit_path = os.path.join(args.dest, "validation-summary-xunit.xml")
    report = TestReport()
    report.scan_tests(args.source, suite_per_file=args.suite_per_file)
    report.print_log()
    defects = load_defects(args.defects)
    if os.path.isfile(validation_summary_path):
        os.remove(validation_summary_path)
    write_excel(report,
                validation_summary_path,
                report_all_tests=args.report_all_tests,
                defects=defects)
    with open(xunit_path, "w", encoding="utf8") as xml_file:
        xunit_report = report.as_xunit()
        pretty_xunit_report = pretty_print_xml(xunit_report)
        xml_file.write(pretty_xunit_report)


if __name__ == '__main__':
    run(read_command_line(None))



# /*############################################################################
#   # Copyright (C) 2020 Intel Corporation
#   #
#   # SPDX-License-Identifier: MIT
#   ############################################################################*/
"""
Generate ABI tests.
"""
# pylint: disable=invalid-name

# To run this script:
# 1. Install clang
# 2. Install clang python binding (pip3 install clang)
# 3. export LD_LIBRARY_PATH=<path to clang SO lib>

import argparse
import os
import glob
import sys
import clang.cindex

ignore_list = [
    'mfxEncryptedData', 'mfxSyncPoint', 'mfxSession', 'mfxLoader', 'mfxConfig'
]

structures = {}


def find_structs(node, mfxStruct):
    """Find structures in the headers"""
    if node.kind == clang.cindex.CursorKind.TYPEDEF_DECL:
        sp = node.underlying_typedef_type.spelling
        if sp.startswith(
                "struct"
        ):  # this is trick. Don't know right way to associate typedef with referenced node
            structures[node.displayname] = node.displayname
        if mfxStruct in sp:
            # print(type(node))
            # print(node.underlying_typedef_type.kind)
            fields = []
            for cc in node.walk_preorder():
                if cc.kind == clang.cindex.CursorKind.FIELD_DECL:
                    if "reserved" not in cc.spelling:
                        fields.append(cc.spelling)

            with open("gen__.c", 'w', encoding="utf-8") as gf:
                original_stdout = sys.stdout
                sys.stdout = gf
                print("#include <stddef.h>")
                print("#include <stdio.h>")
                print("#include <mfx.h>")
                print("int main() {")
                print("printf(\"MSDK_STATIC_ASSERT_STRUCT_SIZE(" + mfxStruct +
                      ", %lu)\\n\", sizeof(" + mfxStruct + "));")
                for f in fields:
                    # MSDK_STATIC_ASSERT_STRUCT_OFFSET(mfxInitializationParam, AccelerationMode  ,0)
                    print(
                        "printf(\"        MSDK_STATIC_ASSERT_STRUCT_OFFSET(" +
                        mfxStruct + ", " + f + ", %lu)\\n\", offsetof(" +
                        mfxStruct + ", " + f + "));")
                print("return 0;}")
                sys.stdout = original_stdout

            print("Done")
            sys.exit(0)
    # Recurse for children of this node
    for c in node.get_children():
        find_structs(c, mfxStruct)


parser = argparse.ArgumentParser(
    description=
    'Searches for the structs with missed ABI backward compatibility tests.')
parser.add_argument('-i', '--ifolder', help='Input folder with header files.')
parser.add_argument('-s', '--structure', help='Name of the structure.')

args = parser.parse_args()

for filename in glob.glob(os.path.join(args.ifolder, '*.h')):
    print("Parsing of " + filename)
    index = clang.cindex.Index.create()
    tu = index.parse(filename, ["-DONEVPL_EXPERIMENTAL"])
    find_structs(tu.cursor, args.structure)

print("Structure definition wasn't located: " + args.structure)
sys.exit(-1)



############################################################################
# Copyright (C) Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""commandline like tools"""

import shutil
import os
import sys
from contextlib import contextmanager
import time
import subprocess  # nosec

VERBOSE = False
if 'VERBOSE' in os.environ:
    if os.environ['VERBOSE'] not in ['']:
        VERBOSE = True


def _resolve_path(path):
    """resolve a path name from either a string, or a list of sub-paths"""
    if isinstance(path, list):
        return os.path.join(*path)
    return path


def _escape_cmd_arg(arg):
    """quote/escape and argument for a command line call so that it can
    be safely used even if it has special charaters"""
    if ' ' in arg or '"' in arg:
        return '"' + arg.replace('"', '""') + '"'
    return arg


def log(message):
    """Log activity"""
    if VERBOSE:
        print(f">> {message}", file=sys.stderr)
        sys.stderr.flush()


@contextmanager
def pushd(dst):
    """change working directory"""
    cur_dir = os.getcwd()
    dest = os.path.join(cur_dir, _resolve_path(dst))
    os.chdir(dest)
    log(f'pushd {dest}')
    try:
        yield
    finally:
        log(f'popd -> {cur_dir}')
        os.chdir(cur_dir)


#pylint: disable=invalid-name
def rm(target):
    """delete a file or folder"""
    target = _resolve_path(target)
    if os.path.exists(target):
        # Delete sometimes fails if done immediately, timeout
        # is not great, but allows filesystem settings to stabilize.
        timeout = time.time() + 10
        while time.time() < timeout:
            try:
                if os.path.isfile(target):
                    log(f'rm {target}')
                    os.remove(target)
                    break
                if os.path.isdir(target):
                    log(f'rm -rf {target}')
                    shutil.rmtree(target)
                    break
            except PermissionError:
                time.sleep(1)


#pylint: disable=invalid-name
def md(target):
    """make a folder"""
    target = _resolve_path(target)
    if target and not os.path.exists(target):
        log(f'mkdir -p {target}')
        os.makedirs(target)


#pylint: disable=invalid-name
def cp(src, dest):
    """copy a file or folder"""
    src = _resolve_path(src)
    dest = _resolve_path(dest)
    if os.path.exists(src):
        rm(dest)
        md(os.path.dirname(src))
        if os.path.isfile(src):
            log(f'cp {src} {dest}')
            shutil.copyfile(src, dest)
        elif os.path.isdir(src):
            log(f'cp {src} {dest}')
            shutil.copytree(src, dest)
        else:
            raise RuntimeError("Cannot copy unknown file type")


def join_command(command):
    """Join a series or parameters into a command, escaping if needed"""
    return ' '.join([_escape_cmd_arg(argument) for argument in command])


def run_command(*args, no_throw=False, env=None):
    """Run a command"""
    if len(args) == 1:
        cmd = args[0]
    else:
        cmd = join_command(args)
    log(f'{cmd}')
    if os.name != 'nt':
        cmd = "exec bash -c '" + cmd + "'"
    with subprocess.Popen(cmd, shell=True, env=env) as proc:  # nosec
        proc.communicate()
        if not no_throw and proc.returncode != 0:
            raise RuntimeError("Error running command: " + cmd)
        return proc.returncode


def run_commands(*args, no_throw=False, env=None):
    """Run several commands"""
    commands = []
    for arg in args:
        if isinstance(arg, (str)):
            commands.append(arg)
        else:
            commands.append(join_command(arg))
    if os.name == 'nt':
        script_file = "temp.bat"
    else:
        script_file = "temp.sh"
    with open(script_file, "w", encoding="utf-8") as script:
        log('echo "')
        for cmd in commands:
            log(f'{cmd}')
            script.write(cmd + "\n")
        log(f'" > {script_file}')
    log('{script_file}')
    if os.name == 'nt':
        cmd = script_file
    else:
        cmd = "exec bash -c 'source " + script_file + "'"
    with subprocess.Popen(cmd, shell=True, env=env) as proc:  # nosec
        proc.communicate()
        rm(script_file)
        if not no_throw and proc.returncode != 0:
            raise RuntimeError("Error running: \n" + "\n".join(commands))
        return proc.returncode


def capture_command(*args, env=None):
    """Run a command and capture the output"""
    if len(args) == 1:
        cmd = args[0]
    else:
        cmd = join_command(args)
    log(f'{cmd}')
    if os.name != 'nt':
        cmd = "exec bash -c '" + cmd + "'"
    with subprocess.Popen(  # nosec
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            shell=True,
            env=env) as proc:
        result = proc.communicate()
        return (result[0], result[1], proc.returncode)


def capture_commands(*args, env=None):
    """Run several commands and capture the output"""
    commands = []
    for arg in args:
        if not arg:
            continue
        if isinstance(arg, (str)):
            commands.append(arg)
        else:
            commands.append(join_command(arg))
    script_file = None
    if os.name == 'nt':
        script_file = "temp.bat"
    else:
        script_file = "temp.sh"
    with open(script_file, "w", encoding="utf-8") as script:
        log('echo "')
        for cmd in commands:
            log(f'{cmd}')
            script.write(cmd + "\n")
        log(f'" > {script_file}')
    log('{script_file}')
    if os.name == 'nt':
        cmd = script_file
    else:
        cmd = "exec bash -c 'source " + script_file + "'"
    with subprocess.Popen(  # nosec
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            shell=True,
            env=env) as proc:
        result = proc.communicate()
        rm(script_file)
        return (result[0], result[1], proc.returncode)



# /*############################################################################
#   # Copyright (C) 2020 Intel Corporation
#   #
#   # SPDX-License-Identifier: MIT
#   ############################################################################*/
"""
Check for structures which are not present in ABI tests.
"""
# pylint: disable=invalid-name,consider-using-f-string

# To run this script:
# 1. Install clang
# 2. Install clang python binding (pip3 install clang)
# 3. export LD_LIBRARY_PATH=<path to clang SO lib>

import argparse
import os
import glob
import re
import sys
import clang.cindex

ignore_list = [
    'mfxEncryptedData', 'mfxSyncPoint', 'mfxSession', 'mfxLoader', 'mfxConfig'
]

structures = {}


def find_structs(node):
    """Find structures in the headers"""
    if node.kind == clang.cindex.CursorKind.TYPEDEF_DECL:
        sp = node.underlying_typedef_type.spelling
        if sp.startswith(
                "struct"
        ):  # this is trick. Don't know right way to assotiate typedef with referenced node
            structures[node.displayname] = node.displayname
    # Recurse for children of this node
    for c in node.get_children():
        find_structs(c)


parser = argparse.ArgumentParser(
    description=
    'Searches for the structs with missed ABI backward compatibility tests.')
parser.add_argument('-i', '--ifolder', help='Input folder with header files.')
parser.add_argument('-t', '--test_file', help='Input file with ABI tests.')

args = parser.parse_args()

errCount = 0
state = 'search'
for filename in glob.glob(os.path.join(args.ifolder, '*.h')):
    print("Parsing of " + filename)
    index = clang.cindex.Index.create()
    tu = index.parse(filename, ["-DONEVPL_EXPERIMENTAL"])
    find_structs(tu.cursor)

print(structures)
print("Found structures: {}".format(len(structures)))

print("Parsing of test file: " + args.test_file)

with open(args.test_file, 'r', encoding="utf-8") as file:
    data = file.read()
    for s in structures:
        if not re.search(s, data):
            if s not in ignore_list:
                print("struct {} missed".format(s))
                errCount = errCount + 1

if errCount == 0:
    print("Passed")
else:
    print("Failed with {} errors.".format(errCount))

sys.exit(-errCount)



#!/usr/bin/env python
"""Check to see if results from scorecard exceed minimum required values."""

import argparse
import json
import sys
import yaml


def get_options():
    """Parse command line."""
    description = __doc__.split('\n', maxsplit=1)[0]
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('scorecard',
                        metavar='FILE',
                        help='JSON file containing scorecard')

    parser.add_argument('--config',
                        metavar='FILE',
                        help='YAML file with required minimum scores')

    return parser.parse_args()


def get_scores(scorecard):
    """Extract checks from scorecard as key value pairs"""
    result = {}
    for check in scorecard['checks']:
        result[check['name']] = check['score']
    return result


def main(scorecard, config):
    """Compare scorecard scores with expectations from config.

    Return number of checks that fail to meet expectations.

    """
    num_fails = 0
    checks = config.get('Checks')
    scores = get_scores(scorecard)
    for check, minval in checks.items():
        score = scores.get(check, -1)
        if score < minval:
            num_fails += 1
        print(f"{check:23}: {score:2}/{minval:2}"
              f"{' ':6} {'Passed' if score >= minval  else 'Failed'}")
    return num_fails


if __name__ == "__main__":
    options = get_options()

    with open(options.scorecard, 'r', encoding="utf-8") as json_file:
        score_dict = json.load(json_file)

    config_dict = {'Checks': {}}
    if options.config:
        with open(options.config, 'r', encoding="utf-8") as yaml_file:
            config_dict = yaml.safe_load(yaml_file)

    sys.exit(main(score_dict, config_dict))



#!/usr/bin/env python3
############################################################################
# Copyright (C) 2020 Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""
Post-process xUnit tests.

"""
import argparse
import os
import csv
import defusedxml.ElementTree as DET

## Alternate get testcase ... name +if skipped failure or error


def read_command_line(cmd_line):
    """Read command line arguments"""
    parser = argparse.ArgumentParser(
        description=globals()['__doc__'],
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('xunit_path',
                        metavar="FILE",
                        help="xUnit file to process",
                        type=os.path.abspath,
                        action='store')

    parser.add_argument('-i',
                        dest='include_list',
                        nargs='+',
                        metavar="substring",
                        help="list of test name substrings to include",
                        default=["Double", "Null", "Unsupported", "Invalid"])

    parser.add_argument('-o',
                        dest='csv_path',
                        metavar="csvfile",
                        help="path to the generated output file",
                        default="out.csv")

    args = parser.parse_args(args=cmd_line)

    return args


def read_xunit(xunit_path):
    """Read data from xunit file into dictionary"""
    print("filename:", xunit_path)
    tree = DET.parse(xunit_path)
    root = tree.getroot()
    data = {}
    for test_case in root.findall("testcase"):
        name = ""
        if "classname" in test_case.attrib:
            classname = test_case.attrib["classname"].split(".")
            if classname:
                name += ".".join(classname) + "."
        name += test_case.attrib['name']

        failure = test_case.find("failure")
        if failure is not None:
            data[name] = "failure"
            continue

        error = test_case.find("error")
        if error is not None:
            data[name] = "error"
            continue

        skip = test_case.find("skipped")
        if skip is not None:
            data[name] = "skipped"
            continue

        data[name] = "pass"
    return data


def filter_xunit(data, include_list):
    """Filter xunit data"""
    filtered_data = {
        k: v
        for k, v in data.items()
        if any(substring in k for substring in include_list)
    }
    return filtered_data


def write_csv(data, csv_file):
    """Write data to CSV file"""
    with open(csv_file, 'w', newline='', encoding="utf-8") as csvfile:
        csvwriter = csv.writer(csvfile,
                               delimiter=',',
                               quotechar='|',
                               quoting=csv.QUOTE_MINIMAL)
        for k, result in data.items():
            csvwriter.writerow([k, result])


def run(args):
    """main entry point"""
    raw_data = read_xunit(args.xunit_path)
    filtered_data = filter_xunit(raw_data, args.include_list)
    write_csv(filtered_data, args.csv_path)


if __name__ == '__main__':
    run(read_command_line(None))



"""
This script provides functionality for processing issue tracking data
extracted from JSON and YAML files. It includes the ability to count
issues by severity, filter issues based on configuration, and determine
an overall pass/fail status based on the issues present.
"""
import json
import argparse
import yaml


# Function to load data from a JSON file
def load_json(file_name):
    """
    Load data from a JSON file.

    Args:
        file_name (str): The name of the file to load.

    Returns:
        dict: The data loaded from the JSON file.
    """
    with open(file_name, 'r', encoding="utf-8") as file:
        return json.load(file)


# Function to load data from a YAML file
def load_yaml(file_name):
    """
    Load data from a YAML file.

    Args:
        file_name (str): The name of the file to load.

    Returns:
        dict: The data loaded from the YAML file.
    """
    with open(file_name, 'r', encoding="utf-8") as file:
        return yaml.safe_load(file)


# Function to count the number of high severity issues
def high_count(issues):
    """
    Count the number of high severity issues.

    Args:
        issues (list of dict): A list of issues.

    Returns:
        int: The count of high severity issues.
    """
    return sum(1 for issue in issues if issue['severity'] == 'High')


# Function to count the number of medium severity issues
def medium_count(issues):
    """
    Count the number of medium severity issues.

    Args:
        issues (list of dict): A list of issues.

    Returns:
        int: The count of medium severity issues.
    """
    return sum(1 for issue in issues if issue['severity'] == 'Medium')


# Function to count the number of low severity issues
def low_count(issues):
    """
    Count the number of low severity issues.

    Args:
        issues (list of dict): A list of issues.

    Returns:
        int: The count of low severity issues.
    """
    return sum(1 for issue in issues if issue['severity'] == 'Low')


# Function to count the number of informational severity issues
def info_count(issues):
    """
    Count the number of informational severity issues.

    Args:
        issues (list of dict): A list of issues.

    Returns:
        int: The count of informational severity issues.
    """
    return sum(1 for issue in issues if issue['severity'] == 'Info')


# Retrieve ignored issues from the YAML configuration
def get_ignored_issues(config):
    """
    Retrieve ignored issues from the YAML configuration.

    Args:
        config (dict): A dictionary containing configuration, including ignored issues.

    Returns:
        tuple: A tuple containing lists of ignored issues, ignored severity, and ignored file names.
    """
    ignored_issues = config.get('ignored_issues', [])
    ignored_severity = config.get('ignored_severity', [])
    ignored_file = config.get('ignored_file', [])
    return ignored_issues, ignored_severity, ignored_file


# Function to extract the file name from a given path
def extract_file_name(full_file_name):
    """
    Extracts the third component from a file path.

    Args:
        full_file_name (str): The full file path.

    Returns:
        str: The extracted file name or 'Unknown' if not available.
    """
    parts = full_file_name.split('|')
    return parts[2] if len(parts) >= 3 else 'Unknown'


# Function to determine if an issue should be included based on various criteria
def issues_included(issue_key, severity, ignored_issues, ignored_severities,
                    ignored_file, file_name):
    """
    Determines if an issue should be included based on various criteria.

    Args:
        issue_key (str): The issue key.
        severity (str): The severity of the issue.
        ignored_issues (list): A list of issue keys to ignore.
        ignored_severities (list): A list of severities to ignore.
        ignored_file (list): A list of file names to ignore.
        file_name (str): The file name of the issue.

    Returns:
        bool: True if the issue should be included, False otherwise.
    """
    return (issue_key not in ignored_issues
            and severity not in ignored_severities
            and file_name not in ignored_file)


# Main function to extract issues from JSON data
def get_issues(data, ignored_issues, ignored_severities, ignored_file):
    """
    Extract issues from the parsed JSON data, excluding specific criteria.

    Args:
        data (dict): The parsed data from JSON.
        ignored_issues (list): A list of issue keys to ignore.
        ignored_severities (list): A list of severities to ignore.
        ignored_file (list): A list of file names to ignore.

    Returns:
        list of dict: A list of issue dictionaries.
    """
    issues = []
    for file_key, issue_details in data.items():
        if not file_key.startswith('File'):
            continue

        full_file_name = issue_details.get('File Name', 'Unknown')
        file_name = extract_file_name(full_file_name)

        for key, value in issue_details.items():
            if not (isinstance(value, str) and '[' in value and ']' in value):
                continue

            start = value.find('[') + 1
            end = value.find(']')
            severity_with_info = value[start:end].strip().split(' ')[0]

            if issues_included(key, severity_with_info, ignored_issues,
                               ignored_severities, ignored_file, file_name):
                issues.append({
                    "issue_name": key,
                    "severity": severity_with_info,
                    "file": file_name
                })
    return issues


def pass_or_fail(issues):
    """
    Determine the overall pass/fail status based on the list of issues.

    Args:
        issues (list of dict): A list of issues.

    Returns:
        str: "Pass" if there are no issues, otherwise "Fail".
    """
    # Determine if the result is pass or fail based on remaining issues
    return "Pass" if not issues else "Fail"


def main():
    """
    Main part of the script.
    Parses command-line arguments to obtain JSON and YAML file names,
    loads data from these files, processes it to filter out ignored issues,
    and prints a summary of the analysis including the overall status
    and details of each issue.
    """
    # Create the parser
    parser = argparse.ArgumentParser()

    # Add arguments for the JSON and YAML file names
    parser.add_argument('json_file')
    parser.add_argument('yaml_file')

    # Parse the command-line arguments
    args = parser.parse_args()

    # Use the parsed arguments to get the file names
    json_data = load_json(args.json_file)
    ignore_config = load_yaml(args.yaml_file)

    # Retrieve ignored issues from YAML configuration
    ignored_issues_list, ignored_severities_list, ignored_file_list = get_ignored_issues(
        ignore_config)

    # Get the list of issues while excluding the ignored ones
    issues_list = get_issues(json_data, ignored_issues_list,
                             ignored_severities_list, ignored_file_list)

    # Determine the overall pass or fail status
    result = pass_or_fail(issues_list)
    print("Status:", result)

    # Output the overall status and counts of each severity level
    print("High severity count:", high_count(issues_list))
    print("Medium severity count:", medium_count(issues_list))
    print("Low severity count:", low_count(issues_list))
    print("Info severity count:", info_count(issues_list))

    # Output the details of each issue not ignored
    for issue in issues_list:
        print(f"Issue Name: {issue['issue_name']}; "
              f"Severity: {issue['severity']}; "
              f"File: {issue['file']}")


if __name__ == "__main__":
    main()



"""
Module Description: This script provides a function to upload files for SDL evidence.

Usage:
    python evidence_upload.py --api_key <API_KEY>
    --user_id <USER_ID> --project_id <PROJECT_ID>
    --task_id <TASK_ID> --file_paths <FILE_PATHS> [--label <LABELS>]
"""
import os
import argparse
from urllib.parse import quote, quote_plus
import sys
import requests
import urllib3

urllib3.disable_warnings()


def upload_file(api_key, user_id, project_id, task_id, file_path, labels,
                output_prefix):
    # pylint: disable=too-many-locals
    # pylint: disable=too-many-arguments
    # pylint: disable=too-many-positional-arguments
    """
    Upload a file for a specific task.

    Parameters:
        api_key (str): API key for authentication.
        user_id (str): User ID.
        task_id (str): Task ID.
        project_id (int): Project ID.
        file_path (str): Path to the file to be uploaded.
        labels (list): List of labels or tags for the file upload.
        output_prefix (str): Prefix for output messages.

    """
    # URL encode user-provided values
    user_id_encoded = quote(user_id)
    project_id_encoded = quote_plus(str(project_id))
    task_id_encoded = quote_plus(task_id)

    # Build URL
    base_url = "https://sdl-e.app.intel.com/uploader/v1/evidence/uploads/documents/creators"
    upload_url = (f"{base_url}/{user_id_encoded}"
                  f"?projectId={project_id_encoded}&taskId={task_id_encoded}")

    if labels:
        for label in labels:
            if label:
                upload_url = f"{upload_url}&label={quote_plus(label)}"
    # Set headers
    headers = {
        "apikey": api_key,
    }

    # Prepare files for upload in multipart/form-data format
    try:
        file_name_with_prefix = f"{output_prefix}{os.path.basename(file_path)}"
        with open(file_path, 'rb') as file:
            files = {'file': (file_name_with_prefix, file)}
    except OSError as exception:
        print(f"Failed to open file {file_path}: {exception}")
        sys.exit(1)

    try:
        # Make the request
        response = requests.post(upload_url,
                                 headers=headers,
                                 files=files,
                                 verify=False)
        print(f" Response Content: {response.text}")

        # Check for success
        if response.status_code == 200:
            response_text = response.text
            if 'Documents were uploaded correctly.' in response_text:
                print(f"File for Task {task_id}  uploaded successfully!")
            else:
                print(f"Failed to upload file for Task {task_id}."
                      "Response: {response_text}")
                sys.exit(response.status_code)
        else:
            print(
                f"Failed to upload file  for Task {task_id}. "
                f"Status Code: {response.status_code}, Response: {response.text}"
            )
            sys.exit(response.status_code)

    except requests.RequestException as exception:
        print(f"Request failed for Task {task_id}: {exception}")
        sys.exit(1)


def main():
    """
    Main function for uploading files.

    Command-line Arguments:
        --api_key (str): API key for authentication.
        --user_id (str): User ID.
        --project_id (int): Project ID.
        --task_id (str): Task ID.
        --file_paths (list): List of file paths.
        --label (list, optional): Label or tags for this file upload.
        --output_prefix (str): Prefix for output messages.
    """
    # Create argument parser
    parser = argparse.ArgumentParser(description="Upload files.")

    # Add arguments
    parser.add_argument("--api_key",
                        required=True,
                        help="API key for authentication")
    parser.add_argument("--user_id", required=True, help="User ID")
    parser.add_argument("--project_id",
                        required=True,
                        type=int,
                        help="Project ID")
    parser.add_argument("--task_id", required=True, help="Task ID")
    parser.add_argument("--file_paths",
                        required=True,
                        nargs="+",
                        help="List of file paths")
    parser.add_argument("--label",
                        required=True,
                        nargs="*",
                        help="Label or tags for this file upload")
    parser.add_argument("--output_prefix",
                        required=True,
                        help="Prefix for output messages")

    # Parse command-line arguments
    args = parser.parse_args()

    # Upload files
    for file_path in args.file_paths:
        upload_file(args.api_key, args.user_id, args.project_id, args.task_id,
                    file_path, args.label, args.output_prefix)
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
############################################################################
# Copyright (C) Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""
Build specification.
"""
# pylint: disable=invalid-name,consider-using-f-string

# Main script to build the specification. Derived from:
# github.com/oneapi-src/oneAPI-spec/blob/main/scripts/oneapi.py

import argparse
import glob
import os
import os.path
import sys
import shutil
import subprocess  # nosec B404
from functools import wraps
from os.path import join

sphinx_build = 'sphinx-build'
source_dir = 'source'
build_dir = 'build'
doxygen_dir = 'doxygen'
doxygen_xml = join(doxygen_dir, 'xml', 'index.xml')

indent = 0
cl_args = []


def action(func):
    """Execute and log action"""
    @wraps(func)
    def wrapped(*args, **kwargs):
        global indent  # pylint: disable=W0603
        log('%s: %s' % (
            args[1] if len(args) > 1 and args[1] else wrapped.__name__,
            args[0],
        ))
        indent += 2
        x = func(*args, **kwargs)
        indent -= 2
        return x

    return wrapped


class cd:
    """Context manager for changing the current working directory"""
    def __init__(self, newPath):
        self.savedPath = ""
        self.newPath = os.path.expanduser(newPath)

    def __enter__(self):
        self.savedPath = os.getcwd()
        log('cd ' + self.newPath)
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)


def log(*args, **kwargs):
    """Log message"""
    print(indent * ' ' + ' '.join(map(str, args)), flush=True, **kwargs)


def shell(c):
    """Execute shell command"""
    log(c)
    if cl_args.dry_run:
        return
    subprocess.check_call(c, shell=True)  # nosec B602


def copy(src, dst):
    """Execute copy command"""
    log('cp', src, dst)
    if cl_args.dry_run:
        return
    shutil.copy(src, dst)


def makedirs(path):
    """Execute mkdir command"""
    log('mkdir -p', path)
    if cl_args.dry_run:
        return
    os.makedirs(path)


def sphinx(root, target):
    """Execute sphinx build"""
    if not cl_args.verbose:
        os.environ['LATEXMKOPTS'] = '--silent'
        os.environ['LATEXOPTS'] = '-interaction=nonstopmode -halt-on-error'
    sphinx_args = '-N'
    if not cl_args.verbose:
        sphinx_args += ' -q'
    if not cl_args.serial:
        sphinx_args += ' -j auto'
    if cl_args.a:
        sphinx_args += ' -a'
    if cl_args.n:
        sphinx_args += ' -n'
    if cl_args.W:
        sphinx_args += ' -W'

    shell('%s -M %s %s %s %s' % (
        sphinx_build,
        target,
        join(root, source_dir),
        join(root, build_dir),
        sphinx_args,
    ))


def get_env(var):
    """Get environment variable"""
    return os.environ[var] if var in os.environ else ''


def root_only(root):
    """Check if in root dir"""
    if root != '.':
        sys.exit('Error: Only works from root')


@action
def dockerbuild(root, _target=None):
    """Build image"""
    cd(root)
    copy(join(root, 'requirements.txt'), join(root, 'docker'))
    copy(join(root, 'ubuntu-packages.txt'), join(root, 'docker'))
    copy(join(root, 'install.sh'), join(root, 'docker'))
    shell('docker build'
          ' -f ./doc/spec/docker/Dockerfile.build'
          ' --build-arg http_proxy=%s'
          ' --build-arg https_proxy=%s'
          ' --build-arg no_proxy=%s'
          ' --tag vpl-spec %s' %
          (get_env('http_proxy'), get_env('https_proxy'), get_env('no_proxy'),
           join(root, 'docker')))


@action
def dockerrun(_root, _target=None):
    """Run image"""
    shell('docker run --rm -it'
          ' -e http_proxy=%s'
          ' -e https_proxy=%s'
          ' -e no_proxy=%s'
          ' --user %s:%s'
          ' --volume=%s:/build'
          ' --workdir=/build'
          ' vpl-spec' % (
              get_env('http_proxy'),
              get_env('https_proxy'),
              get_env('no_proxy'),
              os.getuid(),  # pylint: disable=no-member
              os.getgid(),  # pylint: disable=no-member
              os.getcwd(),
          ))


@action
def clean(root, _target=None):
    """Execute sphinx clean"""
    apply_dirs(root, 'clean')
    sphinx(root, 'clean')


def command(root, target):
    """Execute command list"""
    commands[target](root, target)


def apply_dirs(root, target):
    """Execute command list"""
    elements = join(root, 'doc', 'spec')
    if os.path.exists(elements):
        for curr_dir in dirs:
            command(join(elements, curr_dir), target)


def up_to_date(target, deps):
    """Check if up to date"""
    if not os.path.exists(target):
        return False
    for dep in deps:
        if os.path.getmtime(target) < os.path.getmtime(dep):
            return False
    return True


def doxygen_files(root):
    """Get doxyfile"""
    return [join(root, 'Doxyfile')] + glob.glob(join('api', 'vpl', '**'),
                                                recursive=True)


def doxygen(root, _target=None):
    """Execute doxygen build"""
    with cd(root):
        doxyfile = 'Doxyfile'
        if not os.path.exists(doxyfile) or up_to_date(join(root, doxygen_xml),
                                                      doxygen_files(root)):
            return
        shell('doxygen %s' % doxyfile)


@action
def prep(root='.', _target=None):
    """Prepare doxygen build"""
    apply_dirs(root, 'prep')
    doxygen(root)


@action
def build(root, target):
    """Build sphinx"""
    prep(root)
    sphinx(root, target)


def remove_elements(li, elements):
    """Remove elements"""
    for e in elements:
        if e in li:
            li.remove(e)
    return li


@action
def sort_words(_root, _target=None):
    """Sort words in spell check"""
    with open(join('source', 'spelling_wordlist.txt'),
              encoding="utf-8") as fin:
        lines = fin.readlines()
    with open(join('source', 'spelling_wordlist.txt'), 'w',
              encoding="utf-8") as fout:
        for li in sorted(list(set(lines))):
            fout.write(li)


commands = {
    'clean': clean,
    'dockerbuild': dockerbuild,
    'dockerrun': dockerrun,
    'html': build,
    'latexpdf': build,
    'spelling': build,
    'singlehtml': build,
    'prep': prep,
    'sort-words': sort_words,
}

dirs = [
    'vpl',
]


def main():
    """Main specification build"""
    global cl_args  # pylint: disable=W0603
    parser = argparse.ArgumentParser(
        description='Build Intel® Video Processing Library spec.')
    parser.add_argument('action',
                        choices=commands.keys(),
                        default='html',
                        nargs='?')
    parser.add_argument('root', nargs='?', default=join('doc', 'spec'))
    parser.add_argument('--branch')
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--dry-run', action='store_true')
    parser.add_argument('--serial', action='store_true')
    parser.add_argument('-W', action='store_true')
    parser.add_argument('-a',
                        action='store_true',
                        help='sphinx -a (build all files)')
    parser.add_argument('-n',
                        action='store_true',
                        help='sphinx -n (nitpicky mode)')
    cl_args = parser.parse_args()

    commands[cl_args.action](cl_args.root, cl_args.action)


main()



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK
############################################################################
# Copyright (C) Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""
Run Coverity scan.

This tool requires that Coverity be available in the PATH for
full functionality.
"""
# import modules used
import argparse
from argparse import RawTextHelpFormatter
import os
import sys
import json
import xml.etree.ElementTree as ET  # nosec
from xml.dom import minidom  # nosec

from cmdline import run_command, md, capture_command


def pretty_print_xml(root):
    """Format xml for legibility"""
    serial_xml = ET.tostring(root)
    parsed_xml = minidom.parseString(serial_xml)  # nosec
    return parsed_xml.toprettyxml()


def summarize_server_state(issue, test_info):
    """Summarize state on server for an issue"""
    test_info["status"] = ""
    if "stateOnServer" in issue and issue["stateOnServer"]:
        state_on_server = issue["stateOnServer"]
        test_info["cid"] = state_on_server["cid"]
        test_info["stream"] = state_on_server["stream"]
        if "triage" in state_on_server:
            triage = state_on_server["triage"]
            test_info["status"] = triage["action"].lower()
        test_info["external_ref"] = triage["externalReference"]
    else:
        test_info["cid"] = ""
        test_info["stream"] = ""
        test_info["external_ref"] = ""


def summarize_checker(issue, test_info):
    """Summarize checker properties for an issue"""
    if "checkerProperties" in issue and issue["checkerProperties"]:
        checker_properties = issue["checkerProperties"]
        test_info["desc"] = checker_properties["subcategoryLongDescription"]
        test_info["effect"] = checker_properties["subcategoryLocalEffect"]
    else:
        test_info["desc"] = issue["checkerName"]
        test_info["effect"] = issue["type"]


def build_summary(test_info):
    """Generate a summary string based on test info"""
    summary = []
    if test_info.get("desc"):
        summary.append(test_info["desc"])
    if test_info.get("effect"):
        summary.append(test_info["effect"])
    if test_info.get("mergeKey"):
        summary.append(f"Merge Key: {test_info['mergeKey']}")
    if test_info.get("cid"):
        summary.append(f"CID: {test_info['cid']}")
    if test_info.get("external_ref"):
        summary.append(f"see: {test_info['external_ref']}")
    return '\n\n'.join(summary)


def build_message(test_info):
    """Generate a short message based on test info"""
    if test_info.get("external_ref"):
        return test_info.get("external_ref")
    return f"CID: {test_info['cid']}"


class ReportSummary:
    """Summary of coverity report"""
    def __init__(self, report, problem_statuses):
        self.problems = 0
        self.ignored = 0
        self.problem_statuses = problem_statuses
        self.suite = []
        self.summarize_report(report)

    def summarize_issue(self, issue):
        """Summarize a single issue"""
        test_info = {}
        self.suite.append(test_info)
        summarize_server_state(issue, test_info)
        test_info["ignore"] = test_info["status"] not in self.problem_statuses
        if test_info["ignore"]:
            self.ignored += 1
        else:
            self.problems += 1
        summarize_checker(issue, test_info)
        test_info['type'] = issue["checkerName"]
        test_info['file'] = issue["strippedMainEventFilePathname"]
        test_info['line'] = str(issue["mainEventLineNumber"])
        test_info['col'] = str(issue["mainEventColumnNumber"])
        test_info["name"] = ":".join(
            [test_info["file"], test_info["line"], test_info["col"]])
        test_info["mergeKey"] = issue["mergeKey"]
        test_info["summary"] = build_summary(test_info)
        test_info["message"] = build_message(test_info)

    def summarize_report(self, report):
        """summarize a report"""
        if "issues" in report and report["issues"]:
            for issue in report["issues"]:
                self.summarize_issue(issue)


def xunit_from_report(xunit, summary):
    """write an xunit report for the results"""
    testsuites = ET.Element('testsuites')
    testsuite = ET.SubElement(testsuites, "testsuite")
    testsuite.attrib["name"] = "coverity"
    suite_fail = 0
    for test in summary.suite:
        testcase = ET.SubElement(testsuite, "testcase")
        testcase.attrib["classname"] = test['type']
        testcase.attrib["name"] = test["name"]
        if test["ignore"]:
            testcase.attrib["status"] = "Skip"
            skipped = ET.SubElement(testcase, "skipped")
            skipped.text = test["summary"]
            skipped.attrib["message"] = test["message"]
        else:
            suite_fail += 1
            testcase.attrib["status"] = "Fail"
            failure = ET.SubElement(testcase, "failure")
            failure.attrib["type"] = test['type']
            failure.text = test["summary"]
            failure.attrib["message"] = test["message"]
    testsuite.attrib["tests"] = str(len(summary.suite))
    testsuite.attrib["failures"] = str(summary.problems)
    testsuites.attrib["tests"] = str(len(summary.suite))

    pretty_xunit_report = pretty_print_xml(testsuites)
    with open(xunit, "w", encoding="utf-8") as xml_file:
        xml_file.write(pretty_xunit_report)


def read_command_line(cmd_line):
    """
    Read command line arguments
    """

    # Now read full arguments
    parser = argparse.ArgumentParser(description=globals()['__doc__'],
                                     formatter_class=RawTextHelpFormatter)
    parser.add_argument('--version',
                        action='version',
                        version='%(prog)s 0.2.0')
    parser.add_argument('--url',
                        action="store",
                        required=True,
                        help='Coverity Connect Server URL')
    parser.add_argument('--auth-key-file',
                        action="store",
                        default=None,
                        help='Authentication key file.')
    parser.add_argument('--user',
                        action="store",
                        default=None,
                        help='User name.')
    parser.add_argument('--email',
                        action="store",
                        required=True,
                        help='User email.')
    parser.add_argument('--password', default=None, help='User password.')
    parser.add_argument('--stream',
                        action="store",
                        required=True,
                        help='Coverity target stream')
    parser.add_argument('--project',
                        action="store",
                        required=True,
                        help='Coverity target project')
    parser.add_argument('--project-name',
                        action="store",
                        required=True,
                        help='Project Name')
    parser.add_argument('--bu',
                        action="store",
                        default="",
                        help='Business unit (for reports)')
    parser.add_argument(
        '--report',
        action="store_true",
        help='Force reporting, even if no new issues were found')
    #build options
    parser.add_argument('--dir',
                        dest='intermediate_dir',
                        action='store',
                        required=True,
                        help="Intermediate directory")

    parser.add_argument('--strip-path',
                        action='store',
                        required=True,
                        help="Path to remove from hits")
    parser.add_argument('--description',
                        action='store',
                        default="",
                        help="Snapshot description")
    parser.add_argument('--report-dir',
                        action='store',
                        required=True,
                        help="Report Directory")
    parser.add_argument('--code-version',
                        action='store',
                        required=True,
                        help="Code version")
    parser.add_argument('command',
                        help="Build command",
                        nargs=argparse.REMAINDER,
                        action='store')

    # parse arguments (will exit here on invalid args or help)
    args = parser.parse_args(args=cmd_line)

    # Resolve settings form arguments
    if args.auth_key_file:
        args.auth_key_file = os.path.abspath(args.auth_key_file)
    args.intermediate_dir = os.path.abspath(args.intermediate_dir)
    args.report_dir = os.path.abspath(args.report_dir)
    args.strip_path = os.path.abspath(args.strip_path)
    return args


def configure_coverity():
    """Configure Coverity"""
    if os.name == "nt":
        run_command("cov-configure", "--msvc")
    run_command("cov-configure", "--gcc")
    run_command("cov-configure", "--template", "--compiler", "cc",
                "--comptype", "gcc")
    run_command("cov-configure", "--template", "--compiler", "c++",
                "--comptype", "g++")


def build_under_coverity(intermediate_dir, command, strip_path):
    """Build project under coverity"""
    run_command("cov-build", "--dir", intermediate_dir, *command)
    # Analyze
    run_command("cov-analyze", "--dir", intermediate_dir, "--strip-path",
                strip_path, "--enable-constraint-fpp", "--ticker-mode", "none",
                "--disable-default", "--concurrency", "--security", "--rule",
                "--enable-fnptr", "--enable-virtual", "--enable",
                "SECURE_CODING")


# pylint: disable=too-many-arguments,too-many-positional-arguments
# pylint: disable=too-many-locals
def get_preview_report(intermediate_dir,
                       url,
                       stream,
                       code_version,
                       strip_path,
                       preview_report_v2_path,
                       full_report_v9_path,
                       active_report_v9_path,
                       full_html_report_path,
                       active_html_report_path,
                       text_output=None,
                       user=None,
                       password=None,
                       auth_key_file=None):
    """Generate preview reports (Pull from Coverity Connect, but don't push)"""
    env = os.environ.copy()
    if password is not None:
        env["COVERITY_PASSPHRASE"] = password
    if user is not None:
        env["COV_USER"] = user
    args = [
        "--dir",
        intermediate_dir,
        "--url",
        url,
        "--stream",
        stream,
        "--ticker-mode",
        "none",
        "--version",
        code_version,
        "--strip-path",
        strip_path,
        "--preview-report-v2",
        preview_report_v2_path,
    ]
    if auth_key_file:
        args.extend([
            "--auth-key-file",
            auth_key_file,
        ])
    run_command("cov-commit-defects", *args, env=env)

    run_command("cov-format-errors", "--dir", intermediate_dir,
                "--no-default-triage-filters", "--preview-report-v2",
                preview_report_v2_path, "--json-output-v9",
                full_report_v9_path)

    run_command("cov-format-errors", "--dir", intermediate_dir,
                "--preview-report-v2", preview_report_v2_path,
                "--json-output-v9", active_report_v9_path)

    md(full_html_report_path)
    run_command("cov-format-errors", "--dir", intermediate_dir,
                "--no-default-triage-filters", "--preview-report-v2",
                preview_report_v2_path, "--html-output", full_html_report_path)

    md(active_html_report_path)
    run_command("cov-format-errors", "--dir", intermediate_dir,
                "--preview-report-v2", preview_report_v2_path, "--html-output",
                active_html_report_path)

    if text_output:
        with open(text_output, 'w', encoding="utf-8") as dest:
            dest.write(
                capture_command("cov-format-errors", "--dir", intermediate_dir,
                                "--preview-report-v2", preview_report_v2_path,
                                "--triage-attribute-regex", "action",
                                "Undecided", "--text-output-style",
                                "multiline")[0])


# pylint: disable=too-many-arguments,too-many-positional-arguments
def publish_to_coverity_connect(intermediate_dir,
                                url,
                                stream,
                                code_version,
                                strip_path,
                                description,
                                snapshot_id_path,
                                user=None,
                                password=None,
                                auth_key_file=None):
    """Publish to Coverity Connect"""
    env = os.environ.copy()
    if password is not None:
        env["COVERITY_PASSPHRASE"] = password
    if user is not None:
        env["COV_USER"] = user
    args = [
        "--dir",
        intermediate_dir,
        "--url",
        url,
        "--stream",
        stream,
        "--ticker-mode",
        "none",
        "--version",
        code_version,
        "--strip-path",
        strip_path,
        "--description",
        description,
        "--snapshot-id-file",
        snapshot_id_path,
    ]
    if auth_key_file:
        args.extend([
            "--auth-key-file",
            auth_key_file,
        ])
    run_command("cov-commit-defects", *args, env=env)


def write_reports(args, snapshot_id):
    """Write PDF reports"""
    cfg = f"""# This is an example configuration file for Coverity report generators. It
# tells report generators how to generate reports. You can make and modify a
# copy of it for use in configuring a report generator.
#
# Its initial sections apply to all report generators. Later sections,
# marked with "#######" comments, are intended for specific report
# generators.
#
# This file is In YAML format, version 1.2. See
# https://yaml.org/spec/1.2/spec.html for documentation on the format.
# If you simply modify a copy of this file you probably do not need to
# know details about YAML, however, be aware that structure is
# determined by indentation, i.e., the number of spaces at the beginning
# of the line. Multi-line text should be placed inside double quotes ("").
#
# The YAML constructs used here are "mappings" or key: value pairs. Some
# mappings' values are file pathnames. Pathnames may use a slash or
# backslash, whichever is appropriate for the platform, as a separator.
# (Note that according to YAML's quoting rules, backslashes are not
# special unless they are in a character sequence enclosed in double
# quotes.) A relative pathname here is interpreted as relative to the
# directory containing this configuration file. If the configuration did
# not come from a file (e.g., was read on standard input), then the
# pathname would be relative to the report generator process's working
# directory.
#

################## Sections that apply to all reports #############
# Describes information needed in interpreting the rest of the config.
version:   
    # The version of this file's schema.
    # Mandatory field
    schema-version: 6

# Describes settings needed for connecting to Coverity Connect.
connection:   
    # The URL of the Coverity Connect instance.
    # Mandatory field
    url: {args.url}

    # Coverity Connect username. Password or other authentication key
    # is passed in to the application from the command line.
    # Mandatory field
    username: {args.user}

    # Path to an optional file containing additional CA certificates for
    # use in establishing a secure HTTPS connection. These must be in PEM
    # format.
    ssl-ca-certs: 

# The name of the Coverity Connect project.
# Mandatory field
project: "{args.project}"

# Describes the fields in the title page of the report.
title-page:   
    # Name of your company
    # Mandatory field
    company-name: "Intel"

    # Name of the software development project. May be distinct from the
    # Coverity Connect project name.
    # Mandatory field
    project-name: "{args.project_name}"

    # e.g. project-version: v.1.2.3
    # Mandatory field
    project-version: {args.code_version}

    # Optional path to a logo file for your company. Valid image types
    # are bmp, gif, jpg, and png. The maximum allowed image size is 210
    # pixels wide by 70 pixels high
    # Note that backslash characters in a path must be doubled
    # e.g. logo: C:\\logo\\ourlogo.jpg
    # e.g. logo: /var/logo/ourlogo.png
    logo: 

    # Name of your division, group, team or other organizational unit.
    # Mandatory field
    organizational-unit-name: "IPAS"

    # Organizational unit term (e.g., division, group, team).
    # Mandatory field
    organizational-unit-term: "{args.bu}"

    # Name of the entity for which the report was prepared.
    # This is also known as project-contact-name for
    # CIR, CVSS, PCIDSS, MobileOwasp, and Owasp2017
    # Mandatory field
    prepared-for: "{args.user}"

    # Project contact email address. It is used for
    # CIR, CVSS, PCIDSS, MobileOwasp, and Owasp2017
    # Mandatory field
    project-contact-email: "{args.email}"

    # Name of the entity that prepared the report.
    # Mandatory field
    prepared-by: "{args.user}"

# Locale of the report. Valid values are
#     en_US (English)
#     ja_JP (Japanese)
#     ko_KR (Korean)
#     zh_CN (Simplified Chinese)
# Default is en_US
locale: en_US

# Some reports display information about individual issues. These reports
# bound the number of issues displayed in order to control the size of the
# report. This bound is called the issue cutoff count. It is used for CVSS,
# Security, PCIDSS, MobileOwasp, Owasp2017 and DISA-STIG reports.
# Default is 200. Maximum is 10000 for report.
issue-cutoff-count: 4000

# Used for retrieving the defects of specific snapshot id, instead of using the latest snapshot id of
# all the streams associated with project(default behavior).
# It is not supported for CIR report.
snapshot-id: {snapshot_id}

# The most recent snapshot of each stream in the project whose date is less than or equal to the given date,
# will be used to collect the defects instead of latest snapshot
# Snapshot-id will get the highest priority if both snapshot-id and snapshot-date is present.
# Snapshot date should be in MM/DD/YYYY format
# It is not supported for CIR report.
#snapshot-date: 03/29/2022

# An optional comma-separated list of Coverity Connect issue kinds
# if issue kinds are listed here, the report will include issues only for
# the listed issue kinds.
# Possible values fore Issue Kind :
# Quality
# Security
# example: Quality
# example: Quality, Security
# Default: Include issues for both Quality and Security
issue-kind: SECURITY

# Components specification
# An optional comma-separated list of Coverity Connect component names,
# including component map name.
# If components are listed here, the report will include data only for
# the listed components.
# example: Default.lib,Default.src
components: 

################## CERT report #############
cert-report:   
    # Used for CERT config for target level. Valid values are
    # F ==> Fully Compliant
    # L2 ==> L2 Compliant
    # L1 ==> L1 Compliant
    # Default is F
    target-level: L1

################## Synopsys Software Integrity Report #############
ssir-report:   
    # analysis date should be in MM/DD/YYYY format
    # Mandatory field
    analysis-date: 

    # Optional legal text to display in the report
    # e.g:
    #     "This is the first line of multiline legal text
    #     and this is the second line."
    #
    legal-text: ""

################## Coverity Integrity Report #############
cir-report:   
    # Project-related fields
    # The project-description defaults to its description in Coverity
    # Connect, if any.
    # e.g. project-details: Some details for this project
    #      project-description: A short description of the project
    project-description: ""
    project-details: ""

    # Target integrity level.
    # 1 ==> < 1 defect per thousand lines of code
    # 2 ==> < .1 defects per thousand lines of code
    # 3 ==> < .01 defects per thousand lines of code, and other requirements
    # Default is 1
    target-integrity-level: 

    # Name of the highest severity value.
    # Default is "Major".
    high-severity-name: 

    # Name of the unspecified severity value.
    # Default is "Unspecified".
    unspecified-severity-name: 

    # Trial flag. Use "true" if page 3 of the report should not be generated.
    # Page 3 contains severity data which is not relevant for projects that
    # do not use severity.
    # Default is false
    trial: 

    # LOC multiplier for the number of lines of code that have been
    # inspected.
    # Default is 1
    loc-multiplier:

    # Whether to include Low impact defects for calculating the defect density
    # Possible values can be true, false
    # Default is false
    include-low-impact:

################## Coverity Security Report #############
security-report:   
    # There are four Assurance Levels, representing Security Scores of greater
    # than or equal to 60, 70, 80, and 90. When choosing the Assurance Level,
    # consider the potential for damage to life, property, or reputation.
    # An application with high damage potential should have a
    # high Assurance Level.
    # Default is 90 without plugin
    # if plugin yaml is available in the environment, values will come from plugin
    # and the highest score will be the default
    assurance-level-score: 90

    # A level indicating the minimum acceptable score for the report to
    # be considered passing.
    # Possible values can be (AL1, AL2, AL3, AL4) without plugin
    # and default will be AL1, if plugin yaml is available in the environment, values will come from plugin
    # and the highest score value will be the default
    assurance-level: AL1

    # The name of the set of severity mappings used to determine the score
    # of each issue. See the documentation for a description of the severity
    # mapping. The first three mappings are built-in. "Custom" indicates
    # that the mapping identified by "custom-severity-mapping" and
    # "severity-mapping-description" should be used.
    # Valid values:
    #    Carrier Grade
    #    Web application
    #    Desktop application
    #    Custom
    # Default : Carrier Grade, if plugin yaml is available in the environment, values will come from plugin
    # and the first severity-mapping will be the default
    severity-mapping: Carrier grade

    # Optional descriptive text for custom severity mapping
    severity-mapping-description: ""

    # If severity-mapping is Custom then set the below severity map
    custom-severity-mapping:   
        # Possible values are
        #    very high
        #    high
        #    medium
        #    low
        #    very low
        #    informational
        # Default is very high
        modify-data: Very High
        read-data: Very High
        dos-unreliable-execution: Very High
        dos-resource-consumption: Very High
        execute-unauthorized-code: Very High
        gain-privileges: Very High
        bypass-protection-mechanism: Very High
        hide-activities: Very High

disa-stig:   
    # DISA-STIG version
    # default version: V4

    version: V5
"""
    with open("_covreport.yml", "w", encoding="utf-8") as file:
        file.write(cfg)

    # Special environment with Coverity user information
    env = os.environ.copy()
    if args.password:
        env["COVERITY_PASSPHRASE"] = args.password
    if args.user:
        env["COV_USER"] = args.user

    report_path = os.path.join(args.report_dir, "cvss_report.pdf")
    arguments = [
        "_covreport.yml",
        "--output",
        report_path,
        "--report",
    ]
    if args.password:
        arguments.extend([
            "--password",
            "env:COVERITY_PASSPHRASE",
        ])
    run_command("cov-generate-cvss-report", *arguments, env=env)
    report_path = os.path.join(args.report_dir, "security_report.pdf")
    arguments = [
        "_covreport.yml",
        "--output",
        report_path,
    ]
    if args.password:
        arguments.extend([
            "--password",
            "env:COVERITY_PASSPHRASE",
        ])
    run_command("cov-generate-security-report", *arguments, env=env)


def run(args):
    """main entry point
    """
    # Set up analysis environment
    configure_coverity()
    md(args.intermediate_dir)
    build_under_coverity(args.intermediate_dir, args.command, args.strip_path)

    # Gather data
    md(os.path.join(args.report_dir, "json"))
    preview_report_v2_path = os.path.join(args.report_dir, "json",
                                          "preview_report_v2.json")
    json_report_path = os.path.join(args.report_dir, "json",
                                    "errors_v9_full.json")
    get_preview_report(args.intermediate_dir,
                       args.url,
                       args.stream,
                       args.code_version,
                       args.strip_path,
                       preview_report_v2_path,
                       json_report_path,
                       os.path.join(args.report_dir, "json",
                                    "errors_v9_active.json"),
                       os.path.join(args.report_dir, "html_full"),
                       os.path.join(args.report_dir, "html_active"),
                       os.path.join(args.report_dir, "text_report.txt"),
                       user=args.user,
                       password=args.password,
                       auth_key_file=args.auth_key_file)

    error_count = 0
    with open(json_report_path, "r", encoding="utf-8") as json_file:
        report = json.load(json_file)
        summary = ReportSummary(report, ['undecided'])
        error_count = summary.problems
        xunit_from_report(os.path.join(args.report_dir, "xunit.xml"), summary)
    snapshot_id = None
    if args.report or error_count > 0:
        publish_to_coverity_connect(args.intermediate_dir,
                                    args.url,
                                    args.stream,
                                    args.code_version,
                                    args.strip_path,
                                    args.description,
                                    "_snapshot_id.txt",
                                    user=args.user,
                                    password=args.password,
                                    auth_key_file=args.auth_key_file)

        with open("_snapshot_id.txt", "r",
                  encoding="utf-8") as snapshot_id_file:
            snapshot_id = snapshot_id_file.read().strip()
    snapshot_info = {}
    if snapshot_id is not None:
        snapshot_info["id"] = snapshot_id
    snapshot_info["stream"] = args.stream
    snapshot_info["version"] = args.code_version
    with open(os.path.join(args.report_dir, "json", "info.json"),
              "w",
              encoding="utf-8") as info_file:
        json.dump(snapshot_info, info_file, indent=2)

    if snapshot_id is not None:
        write_reports(args, snapshot_id)
    return error_count


# pylint: disable=bare-except
if __name__ == '__main__':
    sys.exit(run(read_command_line(None)))



#!/usr/bin/env python3
############################################################################
# Copyright (C) Intel Corporation
#
# SPDX-License-Identifier: MIT
############################################################################
"""
Common configuration for building specification.
"""
# pylint: disable=invalid-name

# Derived from:
# github.com/oneapi-src/oneAPI-spec/blob/main/source/conf/common_conf.py

import string
import docutils

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
    'sphinx.ext.graphviz',
    'sphinxcontrib.spelling',
    'sphinxcontrib.inkscapeconverter',
    'sphinxmark',
    'sphinx-prompt',
    'sphinx_substitution_extensions',
    'sphinxcontrib.plantuml',
    'breathe',
]

env = {
    'vpl_spec_version': "2.14.0",
}

prolog_template = string.Template(r"""

.. |vpl_full_name| replace:: Intel\ :supsub:`reg` Video Processing Library
.. |vpl_short_name| replace:: Intel\ :supsub:`reg` VPL
.. |vpl_version| replace:: $vpl_spec_version
.. include:: <isonum.txt>
.. |regsup| replace:: :supsub:`reg`
.. |intel_r| replace:: Intel\ :supsub:`reg`
.. |msdk_full_name| replace:: Intel\ :supsub:`reg`""" +
                                  ' Media Software Development Kit')

rst_prolog = prolog_template.substitute(env)

# for substitutions in code blocks and sphinx-prompts:
substitutions = []

primary_domain = 'cpp'

latex_elements = {
    'preamble': r'''
\usepackage{makeidx}
\usepackage[columns=1]{idxlayout}
\makeindex
\DeclareUnicodeCharacter{2208}{$\in$}
\DeclareUnicodeCharacter{2212}{$-$}
\DeclareUnicodeCharacter{222A}{$\cup$}
\DeclareUnicodeCharacter{2236}{$\colon$}
\DeclareUnicodeCharacter{2260}{$\neq$}
\DeclareUnicodeCharacter{2264}{$\leq$}
\DeclareUnicodeCharacter{2265}{$\geq$}
\DeclareUnicodeCharacter{3B3}{$\gamma$}
\DeclareUnicodeCharacter{39B}{$\Lambda$}
\DeclareUnicodeCharacter{3A3}{$\Sigma$}
\DeclareUnicodeCharacter{3A6}{$\phi$}
\DeclareUnicodeCharacter{3B1}{$\alpha$}
\DeclareUnicodeCharacter{3B2}{$\beta$}
\DeclareUnicodeCharacter{3B4}{$\delta$}
\DeclareUnicodeCharacter{3BB}{$\lambda$}
\DeclareUnicodeCharacter{3BC}{$\mu$}
\DeclareUnicodeCharacter{3BD}{$\nu$}
\DeclareUnicodeCharacter{3C0}{$\pi$}
\DeclareUnicodeCharacter{3C3}{$\sigma$}
\DeclareUnicodeCharacter{FB01}{$fi$}
\DeclareUnicodeCharacter{FB02}{$fl$}
\newcommand{\src}{\operatorname{src}}
\newcommand{\srclayer}{\operatorname{src\_layer}}
\newcommand{\srciter}{\operatorname{src\_iter}}
\newcommand{\srciterc}{\operatorname{src\_iter\_c}}
\newcommand{\weights}{\operatorname{weights}}
\newcommand{\weightslayer}{\operatorname{weights\_layer}}
\newcommand{\weightsiter}{\operatorname{weights\_iter}}
\newcommand{\weightspeephole}{\operatorname{weights\_peephole}}
\newcommand{\weightsprojection}{\operatorname{weights\_projection}}
\newcommand{\bias}{\operatorname{bias}}
\newcommand{\dst}{\operatorname{dst}}
\newcommand{\dstlayer}{\operatorname{dst\_layer}}
\newcommand{\dstiter}{\operatorname{dst\_iter}}
\newcommand{\dstiterc}{\operatorname{dst\_iter\_c}}
\newcommand{\diffsrc}{\operatorname{diff\_src}}
\newcommand{\diffsrclayer}{\operatorname{diff\_src\_layer}}
\newcommand{\diffsrciter}{\operatorname{diff\_src\_iter}}
\newcommand{\diffsrciterc}{\operatorname{diff\_src\_iter\_c}}
\newcommand{\diffweights}{\operatorname{diff\_weights}}
\newcommand{\diffweightslayer}{\operatorname{diff\_weights\_layer}}
\newcommand{\diffweightsiter}{\operatorname{diff\_weights\_iter}}
\newcommand{\diffweightspeephole}{\operatorname{diff\_weights\_peephole}}
\newcommand{\diffweightsprojection}{\operatorname{diff\_weights\_projection}}
\newcommand{\diffbias}{\operatorname{diff\_bias}}
\newcommand{\diffdst}{\operatorname{diff\_dst}}
\newcommand{\diffdstlayer}{\operatorname{diff\_dst\_layer}}
\newcommand{\diffdstiter}{\operatorname{diff\_dst\_iter}}
\newcommand{\diffdstiterc}{\operatorname{diff\_dst\_iter\_c}}
\newcommand{\diffgamma}{\operatorname{diff\_\gamma}}
\newcommand{\diffbeta}{\operatorname{diff\_\beta}}
\newcommand{\workspace}{\operatorname{workspace}}
\newcommand{\srcshape}{\operatorname{src\_shape}}
''',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
    'extraclassoptions': 'openany,oneside',
}

imgmath_latex_preamble = r'''
\newcommand{\src}{\operatorname{src}}
\newcommand{\srclayer}{\operatorname{src\_layer}}
\newcommand{\srciter}{\operatorname{src\_iter}}
\newcommand{\srciterc}{\operatorname{src\_iter\_c}}
\newcommand{\weights}{\operatorname{weights}}
\newcommand{\weightslayer}{\operatorname{weights\_layer}}
\newcommand{\weightsiter}{\operatorname{weights\_iter}}
\newcommand{\weightspeephole}{\operatorname{weights\_peephole}}
\newcommand{\weightsprojection}{\operatorname{weights\_projection}}
\newcommand{\bias}{\operatorname{bias}}
\newcommand{\dst}{\operatorname{dst}}
\newcommand{\dstlayer}{\operatorname{dst\_layer}}
\newcommand{\dstiter}{\operatorname{dst\_iter}}
\newcommand{\dstiterc}{\operatorname{dst\_iter\_c}}
\newcommand{\diffsrc}{\operatorname{diff\_src}}
\newcommand{\diffsrclayer}{\operatorname{diff\_src\_layer}}
\newcommand{\diffsrciter}{\operatorname{diff\_src\_iter}}
\newcommand{\diffsrciterc}{\operatorname{diff\_src\_iter\_c}}
\newcommand{\diffweights}{\operatorname{diff\_weights}}
\newcommand{\diffweightslayer}{\operatorname{diff\_weights\_layer}}
\newcommand{\diffweightsiter}{\operatorname{diff\_weights\_iter}}
\newcommand{\diffweightspeephole}{\operatorname{diff\_weights\_peephole}}
\newcommand{\diffweightsprojection}{\operatorname{diff\_weights\_projection}}
\newcommand{\diffbias}{\operatorname{diff\_bias}}
\newcommand{\diffdst}{\operatorname{diff\_dst}}
\newcommand{\diffdstlayer}{\operatorname{diff\_dst\_layer}}
\newcommand{\diffdstiter}{\operatorname{diff\_dst\_iter}}
\newcommand{\diffdstiterc}{\operatorname{diff\_dst\_iter\_c}}
\newcommand{\diffgamma}{\operatorname{diff\_\gamma}}
\newcommand{\diffbeta}{\operatorname{diff\_\beta}}
\newcommand{\workspace}{\operatorname{workspace}}
\newcommand{\srcshape}{\operatorname{src\_shape}}
'''

mathjax3_config = {
    'tex': {
        'macros': {
            'src': '\\operatorname{src}',
            'srclayer': '\\operatorname{src\\_layer}',
            'srciter': '\\operatorname{src\\_iter}',
            'srciterc': '\\operatorname{src\\_iter\\_c}',
            'weights': '\\operatorname{weights}',
            'weightslayer': '\\operatorname{weights\\_layer}',
            'weightsiter': '\\operatorname{weights\\_iter}',
            'weightspeephole': '\\operatorname{weights\\_peephole}',
            'weightsprojection': '\\operatorname{weights\\_projection}',
            'bias': '\\operatorname{bias}',
            'dst': '\\operatorname{dst}',
            'dstlayer': '\\operatorname{dst\\_layer}',
            'dstiter': '\\operatorname{dst\\_iter}',
            'dstiterc': '\\operatorname{dst\\_iter\\_c}',
            'diffsrc': '\\operatorname{diff\\_src}',
            'diffsrclayer': '\\operatorname{diff\\_src\\_layer}',
            'diffsrciter': '\\operatorname{diff\\_src\\_iter}',
            'diffsrciterc': '\\operatorname{diff\\_src\\_iter\\_c}',
            'diffweights': '\\operatorname{diff\\_weights}',
            'diffweightslayer': '\\operatorname{diff\\_weights\\_layer}',
            'diffweightsiter': '\\operatorname{diff\\_weights\\_iter}',
            'diffweightspeephole': '\\operatorname{diff\\_weights\\_peephole}',
            'diffweightsprojection':
            '\\operatorname{diff\\_weights\\_projection}',  # noqa: E501
            'diffbias': '\\operatorname{diff\\_bias}',
            'diffdst': '\\operatorname{diff\\_dst}',
            'diffdstlayer': '\\operatorname{diff\\_dst\\_layer}',
            'diffdstiter': '\\operatorname{diff\\_dst\\_iter}',
            'diffdstiterc': '\\operatorname{diff\\_dst\\_iter\\_c}',
            'diffgamma': '\\operatorname{diff\\_\\gamma}',
            'diffbeta': '\\operatorname{diff\\_\\beta}',
            'workspace': '\\operatorname{workspace}',
            'srcshape': '\\operatorname{src\\_shape}',
        }
    }
}


def supsub_role(_name,
                _rawtext,
                text,
                _lineno,
                _inliner,
                _options=None,
                _content=None):
    """Superscript substitution"""
    node = docutils.nodes.superscript()
    node2 = docutils.nodes.substitution_reference(refname=text)
    node += [node2]
    return [node], []


def setup(app):
    """Setup"""
    app.add_role('supsub', supsub_role)
    app.add_css_file('custom.css')
